# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8-80 compliant>


bl_info = {
	"name": "Rigs of Rods truckfile format (.truck)",
	"author": "Hans-Georg de Vries",
	"version": (0, 2),
	"blender": (2, 91, 0),
	"location": "File > Import-Export > RoR truck Data (.truck) ",
	"description": "Export Rigs of Rods truck data",
	"warning": "",
	"support_url": "https://forum.rigsofrods.org/threads/blender-ror-truck-exporter.2961/"
	"Scripts/Import-Export/io_mesh_truck",
	"tracker_url": "none",
	"category": "Import-Export"}

if "bpy" in locals():
	import imp
	if "mesh2truck" in locals():
		imp.reload(mesh2truck)


import bpy
from bpy.props import StringProperty, BoolProperty
from bpy_extras.io_utils import ExportHelper

class truckExporter(bpy.types.Operator, ExportHelper):
	"""Save truck data"""
	bl_idname = "mesh2truck.truck"
	bl_label = "Export truck"

	filepath = StringProperty(subtype='FILE_PATH',)
	filename_ext = ".truck"
	projectName = "new_Vehicle"	
	filter_glob = StringProperty(default="*.truck", options={'HIDDEN'})

	# List of operator properties, the attributes will be assigned
	# to the class instance from the operator settings before calling.

	ft_contactable : BoolProperty(
		name="contactable",
		description="mark submesh-faces as collidable with others",
		default=False,
		)

	ft_boat : BoolProperty(
		name="boat",
		description="mark submesh-faces as part of buoyant hull",
		default=False,
		)

	def execute(self, context):
		resetTruckValues(context, self.filepath)
		from . import mesh2truck
		if self.ft_contactable == True: mesh2truck.facetype = "c"
		if self.ft_boat == True: mesh2truck.facetype = "b"
		if self.ft_boat == True and self.ft_contactable == True: mesh2truck.facetype = "D"
		writeDataFile(context, self.filepath)
		return {'FINISHED'}

def resetTruckValues(context,filepath):
	from . import mesh2truck
	mesh2truck.resetValues()

def writeDataFile(context,filepath):
	from . import mesh2truck
	mesh2truck.beginTruckExport(context, filepath)

def menu_export(self, context):
	self.layout.operator(truckExporter.bl_idname, text="RoR truck data (.truck)")

def register():
	bpy.utils.register_class(truckExporter)
	bpy.types.TOPBAR_MT_file_export.append(menu_export)
def unregister():
	bpy.utils.unregister_class(truckExporter)
	bpy.types.TOPBAR_MT_file_export.remove(menu_export)

if __name__ == "__main__":
	register()



