# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8-80 compliant>

"""
This script exports a Mesh to a RoR truck data file.

the truck format is specialized for the vehicles from the game RigsOfRods

Usage:
Execute this script from the "File->Export" menu.

"""

import os
import bpy
import bmesh
import shutil
import numpy
import codecs

# global Values

projectName = bpy.path.basename(os.path.splitext(bpy.context.blend_data.filepath)[0])
facetype = ""
reoNodeslist = []
hasUVData = True
calcMiniMass = True
shocksList="" 
hydrosList=""
expoAllMat = False
beamValue = 0.000000
weightTotal = 1000.000000
controlValue = float(0)

# change/reset gloabl Values

def resetValues():
	global shocksList
	shocksList = ""
	global hydrosList
	hydrosList = ""
	global hasUVData
	hasUVDate=True
	global reoNodeslist
	reoNodeslist.clear()
	global facetype
	facetype = ""
	global beamValue
	beamValue = 0.000000
	global weightTotal
	weightTotal = 1000.000000
	global controlValue
	controlValue = float(0)

def setControlValue(addValue):
	global controlValue
	controlValue = controlValue + addValue

def setWeightTotal(newWeight):
	global weightTotal
	weightTotal = newWeight

def setExportMaterials(boolValue):
	global expoAllMat
	expoAllMat = boolValue

def addShockBeam(v1,v2):
	global shocksList
	shocksList = shocksList + "	"  +str(v1)+",	"+str(v2)+", 40000,   2000,        0.3,       0.3,     1.00\n"

def addHydroBeam(v1,v2):
	global hydrosList
	hydrosList = hydrosList + "	" +str(v1)+",	"+str(v2)+",    0.2\n"

def hasNoUVData():
	global hasUVData
	hasUVData = False

def resetUVDataValue():
	global hasUVData
	hasUVData = True

def calcBeamValue():
	global beamValue
	if getTruckMesh() != None:
		me = getTruckMesh()
		for edge in me.edges:
			beamValue = beamValue + (getBeamLength(edge)*0.001)

# functions

def getBeamLength(edge):
	length = float(0.000000)
	mesh = getTruckMesh()
	if mesh != None:
		node1 = mesh.vertices[edge.vertices[0]]
		node2 = mesh.vertices[edge.vertices[1]]
		p1 = numpy.array([node1.co[0], node1.co[2], node1.co[1]])
		p2 = numpy.array([node2.co[0], node2.co[2], node2.co[1]])

		squared_dist = numpy.sum((p1-p2)**2, axis=0)
		lenght = numpy.sqrt(squared_dist)
	return lenght

def getNodesLoad(nodeID):
	text=", "
	nodeWeight = float(0.000000)
	if getTruckMesh != None:
		mesh = getTruckMesh()
		for edge in mesh.edges:
			if edge.vertices[0] == nodeID or edge.vertices[1] == nodeID:
				nodeWeight = nodeWeight + float(getBeamLength(edge)*0.5)
#		if nodeWeight != 0.000000:
#			nodeWeight = (nodeWeight / beamValue) * (weightTotal*0.001)
#			text = ", l " + str(round(nodeWeight,2))
#		setControlValue(nodeWeight)
	return text

def getLoadValue():
	loadValue = beamvalue * (weightTotal*0.01)
	return loadValue

def ShowMessageBox(message = "", title = "Message Box", icon = 'INFO'):
	def draw(self, context):
		self.layout.label(text=message)

	bpy.context.window_manager.popup_menu(draw, title = title, icon = icon)

def getMaterialName():
	text=""
	try: text = bpy.context.active_object.active_material.name
	except: text="tracks/transred"
	return text

def getImageSourcePath(imageName):
	filepath = "NONE"
	for image in bpy.data.images:
		if image.name == imageName:
			filepath = str(image.filepath_from_user(image_user=None))
	return filepath

def exportAllMaterials(filepath):
	text = ""
	matName = getMaterialName()
	for image in bpy.data.images:
		if image.name != "Render Result":
			sourcePath = getImageSourcePath(image.name)
			destPath = str(filepath.replace(projectName+".truck", image.name))
			if sourcePath is not None: 
				text = text +";	"+ str(sourcePath)+"\n;	"+str(destPath)+"\n\n"
				text = text + getOgreMatText(matName, image.name)
				try: 
					if sourcePath != destPath and os.path.exists(sourcePath) == True: 
						shutil.copyfile(sourcePath, destPath)
				except: 
					ShowMessageBox(message = "can´t find the image source file to copy it to your mod directory! ", title = "Source Image File not found", icon = 'INFO')
	return text

def getOgreMatText(matName, texName):
	text = "material "+matName+"\n{\n    receive_shadows on\n\n"
	text = text + "    technique\n    {\n        pass    " + matName + "\n"
	text = text + "        {\n            texture_unit\n            {\n"
	text = text + "                texture " + texName + "\n"
	text = text + "            }\n        }\n    }\n}\n\n"
	return text

def getHeader(mesh):
	text = ""
	if projectName != "" :
		text = projectName + "\n\n"
	else :
		text = "truckname\n\n"
	text = text+"guid\n\n"
	text = text+"fileformatversion 3\n\n"
	text = text+";author <Art> <AuthorID> <Authorname> <E-Mail>\n"
	text = text+"authors chassis ID YourName eMail\n"
	text = text+"authors texture ID YourName eMail\n"
	text = text+"authors support ID YourName eMail\n\n"
	text = text+"description\n"
	text = text+"	;the first 3 rows will be displayed in truck hud, to display press STRG+T in game\n"
	text = text+"\n"
	text = text+"end_description\n\n"
	text = text+"globals\n"
	text = text+"	;dry mass, cargo mass, material\n"
	text = text+"	" + str(weightTotal*0.5) + ",  " + str(weightTotal*0.5) + ",     "+getMaterialName()+"\n\n"
	text = text+"engine\n"
	text = text+"	;min rpm, max rpm, torque, differential ratio, rear gear, first, second, third, fourth, fifth, sixth ...\n"
	text = text+"	;1000.0,  1500.0, 8000.0,         2.00,   10.85,   10.00, 13.86, 9.52, 6.56, 5.48, 4.58, 3.83, 3.02, 2.53, 2.08, 1.74, 1.43, 1.20, 1.00, -1.0 \n"
	return text

def getFooter():
	text="end\n"
	return text

def getSpecialBeams():
	global hydrosList
	text = "hydros\n"
	text = text + "	;node1, node2, factor, options\n"
	text = text + hydrosList + "\n"

	global shocksList
	text = text + "shocks\n"
	text = text + "	;critical damping=2*sqrt(mass*spring)\n"
	text = text + "	;id1, id2, spring, damping, shortbound, longbound, precomp, options\n"
	text = text + shocksList + "\n"
	return text

def getTruckMesh():
	for ob in bpy.data.objects:
		if ob.type == 'MESH' and ob.data.name == "truck":
			return ob.data

def getTruckObject():
	for ob in bpy.data.objects:
		if ob.type == 'MESH' and ob.data.name == "truck":
			return ob

def getCameras():
	text="cinecam\n"
	for ob in bpy.data.objects:
		if ob.name == "truckCam":
			camPos = ob.location
			text = text + "	; found Camera at Position x: " + str(camPos[0]) +"	z: " + str(camPos[1]) +"	y: " + str(camPos[2]) +"\n"
		if ob.type == 'MESH' and ob.data.name == "truck":
			truckPos = ob.location
			text = text + "	; found Truck  at Position x: " + str(truckPos[0]) +"	z: " + str(truckPos[1]) +"	y: " + str(truckPos[2]) +"\n"
	try:
		text = text + "	;  x,   y,   z, <---------8 bindings--------->, Opt: spring, damping, node-weight\n"
		text = text + "	;"+str(round(camPos[0]-truckPos[0],6)) + ", " + str(round(camPos[2]-truckPos[2],6)) + ", " + str(round(camPos[1]-truckPos[1],6)) + ", "
		text = text + "<---------8 bindings-------->, 8000, 800, 20\n\n"
	except: 
		text = ";  x,   y,   z, <---------8 bindings--------->, Opt: spring, damping, node-weight\n; No Camera found\n"

	text = "cameras\n"
	text = text + "	;center, back, left\n"
	text = text + "	;Center-Node, Back-Node, Left-Node\n"
	return text

def sortNodes(mesh):
	text = ""
	for vertice in mesh.vertices:
		text = str(vertice.index)
		reoNodeslist.append(text)

def triangulatePolygon(poly):
	text = "	cab\n"
	if poly.loop_total == 4:
		text = text + "	" + str(poly.vertices[3]) + ", " + str(poly.vertices[0]) + ", " + str(poly.vertices[1]) + ", " + facetype + "\n"
		text = text + "	" + str(poly.vertices[1]) + ", " + str(poly.vertices[2]) + ", " + str(poly.vertices[3]) + ", " + facetype + "\n"
	elif poly.loop_total == 3:
		text = text + "	" + str(poly.vertices[2]) + ", " + str(poly.vertices[0]) + ", " + str(poly.vertices[1]) + ", " + facetype + "\n"
	else: 
		text = text + "	;Mesh/UV face with more than 4 or less than 3 vertices found - not compatible with rigs of rods submesh system\n" 
	return text

def getFaces():
	text="submesh\n"
	resetUVDataValue()
	me = bpy.context.object.data

	if hasUVData == True:
		try:
			uvData = me.uv_layers.active.data
			for poly in me.polygons:
				text = text +"	texcoords\n"
				for loop_index in range(poly.loop_start, poly.loop_start + poly.loop_total):
					text = text + "	" + str(me.loops[loop_index].vertex_index)+ ", " + str(round(uvData[loop_index].uv[0],6))+", " + str(round(1 - uvData[loop_index].uv[1],6)) +"\n"
				text=text + triangulatePolygon(poly)
		except:
			hasNoUVData()
			ShowMessageBox("For a textured submesh you have to add a material with uv-layout to your modell  ", "Info: - no Material / uv-layer data found!")

	if hasUVData == False:
		text = ";No uv-map data found, generation of texture coordinates skipped!\n"
		for poly in me.polygons:
			text = text + triangulatePolygon(poly)

	return text

def getEdges(mesh):
	text="beams\n"
	for edge in mesh.edges:
		if edge.use_edge_sharp == True:
			addHydroBeam(edge.vertices[0],edge.vertices[1])
		elif edge.use_freestyle_mark == True:
			addShockBeam(edge.vertices[0],edge.vertices[1])
		else:
			text =text + "	" + reoNodeslist[edge.vertices[0]] + ",	" + reoNodeslist[edge.vertices[1]]
			if edge.hide == True:
				text = text + ",	i"
			text = text + "\n"
	return text

def getVertices(mesh):
	sortNodes(mesh)
	text="nodes\n"
	for verts in mesh.vertices:
		nodesData = str(verts.index) +",\t" + str(round(verts.co[0],3))+",\t" + str(round(verts.co[2],3))+",\t" + str(round((verts.co[1]*-1),6))
		text = text + "	" + nodesData
#		if calcMiniMass is True:
#			if getNodesLoad != 0.000000:
#				text = text + getNodesLoad(verts.index)+ "\n"
		text = text +"\n"
#	if calcMiniMass is True: 
#		text = text +"	; Node Load Control-Value : "+ str(round(controlValue,6)) +"\n"
#	else:
#		text = text +"	; Node Load Control-Value : not calculated "+"\n"
		
	return text

def buildtruckData(shape):
	text = ""
	text = text + getHeader(shape) + "\n"
	text = text + getVertices(shape) + "\n"
	text = text + getEdges(shape) + "\n"
	text = text + getCameras() + "\n"
	text = text + getSpecialBeams() + "\n"
	text = text + getFaces() + "\n"
	text = text + getFooter()
	return text

def writeMatFile(context, filepath):
	matFile = getMaterialName();
	matFile = filepath.replace('.truck', '.material')
	f = open(matFile, 'w', encoding='utf-8')
	f.write(exportAllMaterials(filepath))
	f.close()

def writeTruckFile(context, filepath, mesh):
	mesh.validate(verbose=True, clean_customdata=True)
	f = open(filepath, 'w')
	f.write(buildtruckData(mesh))

def beginTruckExport(context, filepath):
	truckFound=False

	if getTruckMesh() != None and getTruckObject() != None :
		try: 
			bpy.ops.object.mode_set(mode = 'OBJECT')
		except: 
			ShowMessageBox("Can´t activate object Mode to export. Make sure you get the 3D view fokused and have left the edit mode  ", "Error! - wrong fokus or no Object Mode")
		calcBeamValue()
		truckFound = True
		bpy.context.view_layer.objects.active = getTruckObject()
		writeTruckFile(context, filepath, getTruckMesh())
		writeMatFile(context, filepath)
	else:
		ShowMessageBox("No truck item found, use the keyword truck as meshname for your truck-object  ", "Error! - no truck-structure found!")





